-----------------------------------------------------------------------------------------
-- Generic functions to add to library
-----------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------
-- Initialize global faction lists
-----------------------------------------------------------------------------------------
function CF_InitFactions()
	print ("UnmappedLands::CF_InitFactions");
	CF_Activity = ActivityMan:GetActivity();
	CF_PlayerTeam = Activity.TEAM_1;
   	CF_CPUTeam = Activity.TEAM_2;
	CF_RogueTeam = Activity.TEAM_3;
	CF_MOIDLimit = 220;

	CF_PlayerFaction = "Dummy";
	CF_CPUFaction = "Dummy";

	CF_MissionEndTimer = Timer();
	CF_StartReturnCountdown = false;
	CF_Activity = ActivityMan:GetActivity();
	
	CF_Factions = {};
	CF_FactionNames = {};
	CF_LightWeaponSets = {};
	CF_HeavyWeaponSets = {};
	CF_ExplosiveSets = {};
	CF_ActorSets = {};
	CF_Probabilities = {};
	CF_InfantryModules = {};	

	CF_BasicActorPrice = 90;
	CF_BasicItemPrice = 90;
	CF_BasicLightWeaponPrice = 60;
	CF_BasicHeavyWeaponPrice = 140;
	CF_BasicExplosivePrice = 12;
	CF_BasicDropshipPrice = 115;	
	
	CF_ArmorCrabs = {}
	CF_ArmorHumans = {}
	
	CF_FactionFiles = CF_ReadFactionsList("UnmappedLands.rte" , "Factions.cfg")

	-- Load factions data
	for i = 1, #CF_FactionFiles do
		print("Loading "..CF_FactionFiles[i])
		f = loadfile(".\\UnmappedLands.rte\\factions\\"..CF_FactionFiles[i])
		f()
	end
	
	print("Factions loaded:")
	for i = 1 , #CF_Factions do
		print(CF_Factions[i])
	end
	
	--Make random factions
	CF_PlayerFaction = CF_Factions[math.random(#CF_Factions)]
	CF_CPUFaction = CF_Factions[math.random(#CF_Factions)]
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_MakeWeapon(faction , islight)
	--print ("CF_MakeWeapon");

	local weapon;
	local r1 , r2;
	local price;
	
	weapon = nil;
	r1 = math.random(#CF_LightWeaponSets[faction]);
	r2 = math.random(#CF_HeavyWeaponSets[faction]);
	
	if islight then
		weapon = CreateHDFirearm(CF_LightWeaponSets[faction][r1] , CF_InfantryModules[faction]);
		price = CF_BasicLightWeaponPrice;
	else
		weapon = CreateHDFirearm(CF_HeavyWeaponSets[faction][r2] , CF_InfantryModules[faction]);
		price = CF_BasicHeavyWeaponPrice;
	end
	
	return weapon , price;
end
-----------------------------------------------------------------------------------------
-- Make explosive. faction - actor faction number
-----------------------------------------------------------------------------------------
function CF_MakeExplosive(faction)
	--print ("CF_MakeExplosive");
	local weapon;
	local r;
	
	weapon = nil;
	r = math.random(#CF_ExplosiveSets[faction]);
	
	weapon = CreateTDExplosive(CF_ExplosiveSets[faction][r] , CF_InfantryModules[faction]);
	
	return weapon , CF_BasicExplosivePrice;
end
-----------------------------------------------------------------------------------------
-- Spawn brain of selected faction. Returns actor or nil if custom position is used
-----------------------------------------------------------------------------------------
function CF_SpawnBrain(team , pos , faction , aimode)
	print ("CF_SpawnBrain");
	local actor = nil;
	local r;
	local weapon;
	
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		actor = CreateAHuman("Brain Robot" , "Base.rte");

		-- Give brain one light weapon
		weapon , price = CF_MakeWeapon(faction , true);
		actor:AddInventoryItem(weapon);
		
		actor.AIMode = aimode;
		actor.Team = team;
		
		if pos ~= nil then
			actor.Pos = pos;
			MovableMan:AddActor(actor);
		else
			return actor;
		end
	end
	
	return nil;
end
-----------------------------------------------------------------------------------------
-- Spawns dropship of selected faction. Returns actor, price.
-----------------------------------------------------------------------------------------
function CF_SpawnDropship(team , pos , faction)
	--print ("CF_SpawnDropship");
	local actor = nil;
	local r;
	local price;
	
	price = 0;
	
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		actor = CreateACDropShip("Drop Ship MK1" , "Base.rte");
		price = CF_BasicDropshipPrice;
		
		actor.AIMode = Actor.AIMODE_DELIVER;
		actor.Team = team;
		actor.Pos = pos;
		actor:SetControllerMode(Controller.CIM_AI, -1);

		return actor , price;
	end
	
	return nil , price;
end
-----------------------------------------------------------------------------------------
-- Spawns armor of selected faction. Returns actor, price.
-----------------------------------------------------------------------------------------
function CF_SpawnArmor(team , pos , faction , aimode)
	--print ("CF_SpawnArmor");
	
	if faction == "$$$random$$$" then
		return CF_SpawnArmor(team , pos, CF_Factions[math.random(#CF_Factions)] , aimode)
	end
	
	local actor = nil;
	local r;
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		if CF_ArmorCrabs[faction] ~= nil then
			r = math.random(#CF_ArmorCrabs[faction]);
			actor = CreateACrab(CF_ArmorCrabs[faction][r], CF_InfantryModules[faction]);
		elseif CF_ArmorHumans[faction] ~= nil then
			print ("Spawn armor actor")
			r = math.random(#CF_ArmorHumans[faction]);
			actor = CreateAHuman(CF_ArmorHumans[faction][r], CF_InfantryModules[faction]);
		else
			actor = CF_SpawnRandomInfantry(team , nil , faction , aimode);
		end
		
		actor.AIMode = aimode;
		actor.Team = team;
		
		if pos ~= nil then
			actor.Pos = pos;
			MovableMan:AddActor(actor);
		else
			return actor , 150;
		end
	end
	
	return nil , 150;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_SpawnRandomInfantry(team , pos , faction , aimode)
	--print ("CF_SpawnRandomInfantry");
	local actor = nil;
	local r;
	local totalprice;
	local price;
	local item;
	
	price = 0;
	totalprice = 0;
	
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		r = math.random(#CF_ActorSets[faction]);
		actor = CreateAHuman(CF_ActorSets[faction][r] , CF_InfantryModules[faction]);
		--print (actor.GoldCost);
		totalprice = totalprice + CF_BasicActorPrice;

		-- probability to give a secondary weapon
		-- which is always light
		if math.random() < CF_Probabilities[faction][2] then
			item , price = CF_MakeWeapon(faction , true);
			actor:AddInventoryItem(item);
			totalprice = totalprice + price;
		end
		
		-- probability to give a heavy weapon
		if math.random() < CF_Probabilities[faction][1] then
			item , price = CF_MakeWeapon(faction , false);
			actor:AddInventoryItem(item);
			totalprice = totalprice + price;
		else
			item , price = CF_MakeWeapon(faction , true);
			actor:AddInventoryItem(item);
			totalprice = totalprice + price;
		end
		
		-- probability to give an explosive
		if math.random() < CF_Probabilities[faction][3] then
			item , price = CF_MakeExplosive(faction);
			actor:AddInventoryItem(item);
			totalprice = totalprice + price;
		end

		-- probability to give a second explosive
		if math.random() < CF_Probabilities[faction][4] then
			item , price = CF_MakeExplosive(faction);
			actor:AddInventoryItem(item);
			totalprice = totalprice + price;
		end
		
		actor.AIMode = aimode;
		actor.Team = team;
		
		if pos ~= nil then
			actor.Pos = pos;
			MovableMan:AddActor(actor);
		else
			return actor , totalprice;
		end
	end
	
	return nil , totalprice;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_SpawnInfantry(givelight , giveheavy , giveexplosive , givedigger , team , pos , faction , aimode)
	--print ("CF_SpawnInfantry");
	local actor = nil;
	local r;
	local totalprice;
	local price;
	local item;
	
	price = 0;
	totalprice = 0;	
	
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		r = math.random(#CF_ActorSets[faction]);
		actor = CreateAHuman(CF_ActorSets[faction][r] , CF_InfantryModules[faction]);
		totalprice = totalprice + CF_BasicActorPrice;
		
		if givelight then
			item , price = CF_MakeWeapon(faction , true);
			actor:AddInventoryItem(item);
			totalprice = totalprice + price;
		end

		if giveheavy then
			item , price = CF_MakeWeapon(faction , false);
			actor:AddInventoryItem(item);
			totalprice = totalprice + price;
		end
		
		-- probability to give an explosive
		if giveexplosive then
			item , price = CF_MakeExplosive(faction);
			actor:AddInventoryItem(item);
			totalprice = totalprice + price;
		end
		
		actor.AIMode = aimode;
		actor.Team = team;

		if pos ~= nil then
			actor.Pos = pos;
			MovableMan:AddActor(actor);
		else
			return actor , totalprice;
		end
	end
	
	return nil , totalprice;
end
-----------------------------------------------------------------------------------------
-- Places garrison troopers on the map
-----------------------------------------------------------------------------------------
function CF_ArrangeGarrisonActors(config)
	print ("CF_ArrangeGarrisonActors");
	local troops;
	
	troops = 5;--config["Planet"..config["LastMissionTerritory"].."Troopers"];
	
	print ("Troops "..troops);

	for i = 1, troops do
		CF_SpawnInfantry(false , true , true , false ,
			CF_PlayerTeam , SceneMan.Scene:GetArea("Garrison"..i):GetCenterPoint() , 
			CF_PlayerFaction , Actor.AIMODE_SENTRY);
	end
end

-----------------------------------------------------------------------------------------
-- Creates experienced elite actor taken from previous battles
-----------------------------------------------------------------------------------------
function CF_SpawnElite(presetnameexp , team , pos , faction , aimode)
	--[[print("CF_SpawnEilte");
	local actor = nil;
	local r;
	local price;
	local presetname , exppoints;
	
	
	presetname , exppoints = CF_ParsePresetName(presetnameexp);

	print(presetname);
	
	price = 0;
	
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		actor = CF_SpawnInfantry(false , true , true , false , team , pos , faction , aimode)
		
		actor.AIMode = aimode;
		actor.Team = team;
		
		if pos ~= nil then
			actor.Pos = pos;
			MovableMan:AddActor(actor);
		else
			return actor , price;
		end
	end
	
	return nil , price;]]--
end
-----------------------------------------------------------------------------------------
-- Deploy elite actors
-----------------------------------------------------------------------------------------
function CF_ArrangeEliteActors(config)
	print ("CF_ArrangeEliteActors");

	for i = 1, tonumber(config["MaxElite"]) do
		if config["Elite"..i] ~= "None" then
			CF_SpawnInfantry(false , true , true , false , 
				CF_PlayerTeam , SceneMan.Scene:GetArea("Elite"..i):GetCenterPoint() , CF_PlayerFaction , Actor.AIMODE_SENTRY);
		end
	end
end
-----------------------------------------------------------------------------------------
-- Replace default dummy actors placed manually to faction-specific actors
-----------------------------------------------------------------------------------------
function CF_ReplaceAddedActors(config)
	print ("CF_ReplaceAddedActors");
	local team;
	local toremove;
	local lpos;
	local viewpoint;
	local colls = {};
    -- Set up AI modes for the Actors that have been added to the scene by the scene definition
    for actor in MovableMan.AddedActors do
		toremove = false; 
		
		if actor.Team ~= CF_RogueTeam then
			print(actor.PresetName);
			-- Replace existing dummy actors with custom ones  
			-- Replace brain units
			if actor.PresetName == "Brain Robot" then
				lpos = Vector(actor.Pos.X , actor.Pos.Y);
				actor.Pos = Vector(0 , - 20);
				toremove = true;
				if actor.Team == CF_PlayerTeam then
					CF_SpawnBrain(CF_PlayerTeam , lpos , CF_PlayerFaction , Actor.AIMODE_SENTRY);
				else
					CF_SpawnBrain(CF_CPUTeam , lpos , CF_CPUFaction , Actor.AIMODE_SENTRY);
				end
			end

			-- Replace soldier units
			if actor.PresetName == "Dummy" then
				lpos = Vector(actor.Pos.X , actor.Pos.Y);
				actor.Pos = Vector(0 , - 20);
				toremove = true;
				if actor.Team == CF_PlayerTeam then
					CF_SpawnRandomInfantry(CF_PlayerTeam , lpos , CF_PlayerFaction , Actor.AIMODE_SENTRY);
				else
					CF_SpawnRandomInfantry(CF_CPUTeam , lpos , CF_CPUFaction , Actor.AIMODE_SENTRY);
				end
			end

			-- Replace armor units
			if actor.PresetName == "Dreadnought" then
				lpos = Vector(actor.Pos.X , actor.Pos.Y);
				actor.Pos = Vector(0 , - 20);
				toremove = true;
				if actor.Team == CF_PlayerTeam then
					CF_SpawnArmor(CF_PlayerTeam , lpos , CF_PlayerFaction , Actor.AIMODE_SENTRY);
				else
					CF_SpawnArmor(CF_CPUTeam , lpos , CF_CPUFaction , Actor.AIMODE_SENTRY);
				end
			end
		end
		
		if toremove then 
			actor.Lifetime = 1
			ActivityMan:GetActivity():ReportDeath(actor.Team,-1)
		end
    end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_GetCPUFaction(config)
	print ("CF_GetCPUFaction");
	faction = config["LastMissionCPUFaction"];
	return faction;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_GetPlayerFaction(config)
	print ("CF_GetPlayerFaction");
	faction = config["PlayerFaction"];
	return faction;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_CountLoot()
	print ("CF_CountLoot");
	local troops;
	local loot;
	
	troops = 0;
	loot = 0;
	
	for actor in MovableMan.Actors do
		if actor.Team == CF_PlayerTeam then
			if actor.PresetName ~= "Brain Robot" and actor.PresetName ~= "Dummy Brain" and
				actor.PresetName ~= "Ronin Brain" then
				troops = troops + 1;
			end
			
			if troops > 5 then
				loot = loot + (actor.Health / 100 * CF_BasicActorPrice);
			end
		end
	end
	
	print ("Actor loot = "..loot);

	for item in MovableMan.Items do
		loot = loot + CF_BasicItemPrice;
	end

	print ("Total loot = "..loot);
	
	return math.ceil(loot);
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_CountRemainingTroopers()
	print ("CF_CountRemainingTroopers");
	local troops;
	
	troops = 0;
	
	for actor in MovableMan.Actors do
		if actor.Team == CF_PlayerTeam then
			if actor.PresetName ~= "Brain Robot" and actor.PresetName ~= "Dummy Brain" and
				actor.PresetName ~= "Ronin Brain" then
				troops = troops + 1;
			end
			
			if troops == 5 then
				break;
			end
		end
	end -- for
	
	return troops;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_UpdateGenericStats(config)
	print ("CF_UpdateGenericStats");

	config["PlayerGold"] = CF_Activity:GetTeamFunds(CF_PlayerTeam);
	config["Kills"] = tonumber(config["Kills"]) + CF_Activity:GetTeamDeathCount(CF_CPUTeam);
	config["Deaths"] = tonumber(config["Deaths"]) + CF_Activity:GetTeamDeathCount(CF_PlayerTeam);
end
-----------------------------------------------------------------------------------------
-- Save most experienced actors data to config
-----------------------------------------------------------------------------------------
function CF_CommitElite(config)
	print ("CF_CommitElite");
	-- Show elite actor marks
	--[[local elitecount;
	local eliteexp = {};
	local eliteactors = {};
	
	elitecount = tonumber(config["MaxElite"]);
	
	for i = 1 , elitecount do
		eliteexp[i] = 0;
		eliteactors[i] = nil;
	end
	
	-- Find toughest elite actors
	for actor in MovableMan.Actors do
		if actor.Team == CF_PlayerTeam then
			local presetname , actorexp;
			local allowed = true;
			
			presetname , actorexp = CF_ParsePresetName(actor.PresetName);

			for i = 1 , #CF_NonEliteActors do
				if presetname == CF_NonEliteActors[i] then
					allowed = false;
				end
			end
			
			if allowed then
				if actorexp ~= nil then
					for j = 1 , elitecount do
						if actorexp >= eliteexp[j] then
							for i = j + 1 , elitecount do
								eliteexp[i] = eliteexp[i - 1];
								eliteactors[i] = eliteactors[i - 1];
							end
							
							eliteexp[j] = actorexp;
							eliteactors[j] = actor;
							break;
						end
					end
				end
			end
		end
	end
	
	for i = 1 , elitecount do
		if eliteactors[i] ~= nil and MovableMan:IsActor(eliteactors[i]) then
			config["Elite"..i] = eliteactors[i].PresetName;
		else
			config["Elite"..i] = "None";
		end
	end]]--
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_InitMission(config)
	print ("CF_InitMission");
	CF_Activity:SetTeamFunds(tonumber(config["PlayerGold"]) , CF_PlayerTeam);
	CF_AIBudget = tonumber(config["LastMissionAIBudget"]);
	CF_PlayerFaction = CF_GetPlayerFaction(config);
	CF_CPUFaction = CF_GetCPUFaction(config);
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_CommitVictory(config , modulename ,  filename)
	if CF_Activity.ActivityState ~= Activity.OVER then
		print ("CF_CommitVictory");
		--config["LastMissionResult"] = "Victory";
		--config["LastMissionLootCollected"] = CF_CountLoot();
		--config["Planet"..config["LastMissionTerritory"].."Troopers"] = CF_CountRemainingTroopers();
		--CF_UpdateGenericStats(config);
		--CF_CommitElite(config);
		
		--CF_WriteConfigFile(config , modulename , filename);
		
		for player = 0, CF_Activity.PlayerCount - 1 do
			FrameMan:SetScreenText("VICTORY!!!", player, 200, -1, false);
		end
	end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_CommitRetreat(config , modulename , filename)
	if CF_Activity.ActivityState ~= Activity.OVER then
		print ("CF_CommitRetreat");
		
		--config["LastMissionResult"] = "Retreat";
		--config["LastMissionLootCollected"] = "0";
		--config["Planet"..config["LastMissionTerritory"].."Troopers"] = "0";
		--CF_UpdateGenericStats(config);
		
		-- Clear elite troopers
		--for i = 1 , tonumber(config["MaxElite"]) do
		--	config["Elite"..i] = "None";
		--end		
		
		--CF_WriteConfigFile(config , modulename , filename);

		for player = 0, CF_Activity.PlayerCount - 1 do
			FrameMan:SetScreenText("RETREAT!!!", player, 200, -1, false);
		end
	end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_CommitDefeat(config , modulename , filename)
	if CF_Activity.ActivityState ~= Activity.OVER then
		print ("CF_CommitDefeat");

		--config["LastMissionResult"] = "Defeat";
		--config["LastMissionLootCollected"] = "0"
		--config["Planet"..config["LastMissionTerritory"].."Troopers"] = "0";
		--config["PlayerHasBrain"] = "false";
		--CF_UpdateGenericStats(config);

		-- Clear elite troopers
		--for i = 1 , tonumber(config["MaxElite"]) do
		--	config["Elite"..i] = "None";
		--end		
		
		--CF_WriteConfigFile(config , modulename , filename);

		for player = 0, CF_Activity.PlayerCount - 1 do
			FrameMan:SetScreenText("COMMANDER DEAD! DEFEAT!!!", player, 200, -1, false);
		end
	end
end
-----------------------------------------------------------------------------------------
-- Transfers player to strategy screen after 3 second of victory message
-----------------------------------------------------------------------------------------
function CF_ReturnOnMissionEnd()
	if CF_StartReturnCountdown then
		if CF_MissionEndTimer:IsPastSimMS(3000) then
			print ("Transfer to strategy screen");
			if CF_Activity.ActivityState ~= Activity.OVER then
				ActivityMan:EndActivity();
			end
		end
	end
end
-------------------------------------------------------------------------------
-- Parses given presetname into Preset : Experience tuple
-------------------------------------------------------------------------------
function CF_ParsePresetName(s)
	local pos1 , pos2;
	
	pos = string.find(s ,":");
	
	if pos ~= nil then
		local  preset , exppoints;

		--s = string.gsub(s , "\n" , "");
		preset = string.sub(s , 1 , pos - 1);
		exppoints = string.sub(s , pos + 1 , string.len(s));

		return preset , tonumber(exppoints);
	else
		return s , nil;
	end
end
-----------------------------------------------------------------------------------------
-- Checks if player buys only allowed actors. Gibs prohibited actors if not
-----------------------------------------------------------------------------------------
function CF_CheckForeignTroops()
	-- Comment out contents of this function to disable foreign actor kill

	--[[local isvalid;

	if CF_ForeignCheckTimer:IsPastSimMS(250) and not debugcheats then
		for actor in MovableMan.Actors do
			if actor.Team == CF_PlayerTeam then
				if actor.ClassName ~= "ADoor" then
					isvalid = false;

					local presetname , actorexp;
					presetname , actorexp = CF_ParsePresetName(actor.PresetName);
					
					for i = 1, #CF_AllowedActors[CF_PlayerFaction] do
						if presetname == CF_AllowedActors[CF_PlayerFaction][i] then
							isvalid = true;
						end
					end
					
					if not isvalid then
						actor:GibThis();
					end
				end
			end
		end -- for
		
		CF_ForeignCheckTimer:Reset();
	end--]]--
end
