-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function RoninLastStand:StartActivity()
	print("START! -- RoninLastStand:StartActivity()!");

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	-- self.ConfigName = "ronin.dat";
	self.ConfigName = "current.dat";
	self.ModuleName = "UnmappedLands.rte";

	dofile("./"..self.ModuleName.."/Libraries/CF_ConfigLibrary.lua");
	dofile("./"..self.ModuleName.."/Libraries/CF_MissionLibrary.lua");
	
	self.Config = CF_ReadConfigFile(self.ModuleName , self.ConfigName);
	CF_InitFactions();
	CF_InitMission(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------

	self.EnemyLZ = {}
	self.BasePosts = {};
	self.Bases = {};
	self.BaseTaken = {};
	self.BaseEmpty = {};
	self.BaseAttacked = {};
	self.BaseGarrison = {};
	self.BaseAttackers = {};
	self.BasePostsUnitsSent = {};
	self.IsBasePostEmpty = {};
	
	self.BaseArrowsCoords = {};
	self.BaseArrowsCoords[1] = Vector(4140 , 280);
	self.BaseArrowsCoords[2] = Vector(3230 , 240);
	self.BaseArrowsCoords[3] = Vector(1948 , 320);
	
	for i = 1 , 4 do
		self.EnemyLZ[i] = SceneMan.Scene:GetArea("Base"..i.."LZ");
		self.Bases[i] = SceneMan.Scene:GetArea("Base"..i);
		self.BaseTaken[i] = false;
		self.BaseEmpty[i] = false;
		self.BaseAttacked[i] = false;
		self.BaseGarrison[i] = 0;
		self.BaseAttackers[i] = 0;
		
		self.BasePosts[i] = {};
		self.BasePostsUnitsSent[i] = {};
		self.IsBasePostEmpty[i] = {};
		for j = 1 , 3 do
			self.BasePosts[i][j] = SceneMan.Scene:GetArea("Post"..i.."-"..j);
			self.BasePostsUnitsSent[i][j] = -100000;
			self.IsBasePostEmpty[i][j] = true;
		end
	end

	self.Terminal = SceneMan.Scene:GetArea("Terminal");
	
	self.RetakeLZ = {}
	for i = 1 , 3 do
		self.RetakeLZ[i] = SceneMan.Scene:GetArea("RetakeLZ"..i);
	end


	self.TerminalHacked = false;
	
	self.ChunksComplete = 0;
	self.TotalChunks = 100;
	self.ChunkInterval = 250;
	
	self.ScreenActivated = false;
	
	self.PlayerBrainBase = 0;
	
	self.Screens = {};
	self.Screens[1] = 0;
	self.Screens[2] = 0;

	self.ScreenCoords = {};
	self.ScreenCoords[1] = Vector(3048 , 512);
	self.ScreenCoords[2] = Vector(3120 , 512);
	
	
	-- How much time to wait until trooper reach his assigned position
	self.DeploymentDelay = 20000
	
	
	self.braindead = {};
	self.AssignedBrains = {};
	
	for player = 0, self.PlayerCount - 1 do
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			local brain = CF_SpawnBrain(CF_PlayerTeam , nil , CF_PlayerFaction , Actor.AIMODE_SENTRY);
			-- brain.Pos = Vector(2092 , 564); -- Debug
			brain.Pos = Vector(5376 + player * 30 , 536); -- Gameplay
			brain.AIMode = Actor.AIMODE_SENTRY;
			self.AssignedBrains[player] = brain;

            MovableMan:AddActor(brain);
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
			self:SwitchToActor(brain, player, CF_PlayerTeam);
        end
    end

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReplaceAddedActors(self.Config);
	CF_ArrangeEliteActors(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------
	
    self.FightPhase = { NOFIGHT = 0 , COUNTER = 1};
    self.CurrentFightPhase = self.FightPhase.NOFIGHT;
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();

	self.HackTimer = Timer();
	self.HackTimer:Reset();
	
	self.CounterAttackTimer = Timer();
	self.CounterAttackTimer:Reset();
	
	self.DefaultDropshipSpawnInterval = 16000;
	self.CounterDropshipSpawnInterval = 10000;
	self.DropshipSpawnInterval = self.DefaultDropshipSpawnInterval;

	self.DropshipSpawnTimer = Timer();
	self.DropshipSpawnTimer:Reset();
	
	for actor in MovableMan.AddedActors do
		actor.AIMode = Actor.AIMODE_SENTRY;
	end	
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function RoninLastStand:PauseActivity(pause)
    print("PAUSE! -- RoninLastStand:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function RoninLastStand:EndActivity()
    print("END! -- RoninLastStand:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function RoninLastStand:DistributeReinforcements()
	-- Assign falling troops to specific firing points
	for actor in MovableMan.Actors do
		if actor.Team == CF_CPUTeam and actor.AIMode == Actor.AIMODE_SENTRY then
			for i = 1 , #self.EnemyLZ do
				if self.EnemyLZ[i]:IsInside(actor.Pos) then
					local isassigned;
					isassigned = false;

					-- If base is not under attack move to firing point
					-- else alert teammates and move to first enemy found
					if self.BaseAttacked[i] == false then
						for j = 1 , #self.BasePosts[i] do
						
							if self.IsBasePostEmpty[i][j] and self.GenericTimer.ElapsedSimTimeMS - self.BasePostsUnitsSent[i][j] > self.DeploymentDelay then
								actor.AIMode = Actor.AIMODE_GOTO;
								actor:ClearAIWaypoints();
								
								actor:AddAISceneWaypoint(self.BasePosts[i][j]:GetCenterPoint());
								
								isassigned = true;
								self.BasePostsUnitsSent[i][j] = self.GenericTimer.ElapsedSimTimeMS;
								print("Assigned to "..i.."-"..j)
								break;
							end -- if
						end -- for j
					else
						local enemyfound = nil
					
						for actor2 in MovableMan.Actors do
							if self.Bases[i]:IsInside(actor2.Pos) then
								if actor2.Team == CF_PlayerTeam then
									if enemyfound == nil then
										enemyfound = actor2
										--Debug
										--enemyfound:FlashWhite(100)
									end
								else
									-- Tell the rest of the troops to find enemy
									if enemyfound ~= nil and string.find(actor2.PresetName , 'Brain') then
										if actor2.AIMode == Actor.AIMODE_SENTRY then
											actor2.AIMode = Actor.AIMODE_GOTO;
											actor2:ClearAIWaypoints();
												
											actor2:AddAISceneWaypoint(enemyfound.Pos);
											--Debug
											--actor2:FlashWhite(100)
										end -- if
									end -- if
								end -- if - else
							end -- if
						end -- for

						--Send freshly spawned unit to find intruder
						if enemyfound ~= nil then
							actor.AIMode = Actor.AIMODE_GOTO;
							actor:ClearAIWaypoints();
								
							actor:AddAISceneWaypoint(enemyfound.Pos);
						else
							print ("Enemies not found. Assign as a Hunter!")
							actor.AIMode = Actor.AIMODE_BRAINHUNT;
						end
					end
					
					-- If all points in selected zone are taken, assign to brain hunt
					-- to avoid crowds of sentry troops
					if not isassigned then
						print ("All points occupied. Assign as a Hunter!")
						actor.AIMode = Actor.AIMODE_BRAINHUNT;
					end
				end -- if 
			end -- for i
		end -- if
	end--]]--
end
-----------------------------------------------------------------------------------------
-- Update base defence stats to make dispatch decisions later
-----------------------------------------------------------------------------------------
function RoninLastStand:UpdateUnitStats()
	-- Reset all current states, they are refilled here
	-- Zones are considered non-empty if we have non-expired troops sent to this zone
	
	for i = 1 , #self.IsBasePostEmpty do
		self.BaseAttacked[i] = false;
		self.BaseGarrison[i] = 0;
		self.BaseAttackers[i] = 0;
		for j = 1, #self.IsBasePostEmpty[i] do
			self.IsBasePostEmpty[i][j] = true;
		end
	end

	self.PlayerBrainBase = 0;
	
	local aihascommander = false;
	
	for actor in MovableMan.Actors do
		-- Check for empty firing points
		for i = 1 , #self.Bases do
			if self.Bases[i]:IsInside(actor.Pos) then
				if actor.Team == CF_CPUTeam  then
					if not string.find(actor.PresetName ,"Turret") and not string.find(actor.PresetName ,"SAM Site") then
						self.BaseGarrison[i] = self.BaseGarrison[i] + 1;
						--Debug
						-- self:AddObjectivePoint("Basterd!", actor.AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
					end

					for j = 1 , #self.BasePosts[i] do
						if self.BasePosts[i][j]:IsInside(actor.Pos) then
							self.IsBasePostEmpty[i][j] = false;
						end -- if
					end -- for
				elseif actor.Team == CF_PlayerTeam  then
					self.BaseAttacked[i] = true;
					self.BaseAttackers[i] = self.BaseAttackers[i] + 1;
					
					if string.find(actor.PresetName , "Brain") then
						self.PlayerBrainBase = i;
					end
					
				end
			end
		end
			
		if actor.Team == CF_CPUTeam  then
			if string.find(actor.PresetName ,"Brain") then
				aihascommander = true;
				self:AddObjectivePoint("Kill", actor.AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
			end
		end -- if
	end
	
	--Mark empty bases as taken by enemy
	for i = 1, #self.Bases do
		if self.BaseTaken[i] == false then
			if self.BaseAttackers[i] > 0 and self.BaseGarrison[i] == 0 then
				self.BaseTaken[i] = true
				self:StartCounterAttack();
			end
		end
	end
	
	if self.GenericTimer:IsPastSimMS(500) then
		-- AI commander died, victory
		if not aihascommander then
			if CF_Activity.ActivityState ~= Activity.OVER then
				---------------------------------------------------------------------
				-- Mission framework generic part. Must present in every mission!
				CF_CommitVictory(self.Config , self.ModuleName , self.ConfigName);
				-- end of generic part
				---------------------------------------------------------------------
				self.WinnerTeam = CF_PlayerTeam;
				ActivityMan:EndActivity();
			end
		end
	end--]]--
end
-----------------------------------------------------------------------------------------
-- Process hacking routines
-----------------------------------------------------------------------------------------
function RoninLastStand:HackTerminal()
	local isinside = false;

	-- Show boot completion mark
	for player = 0, self.PlayerCount - 1 do
		if MovableMan:IsActor(self:GetPlayerBrain(player)) then
			if self.Terminal:IsInside(self:GetPlayerBrain(player).Pos) then
				isinside = true;
			
				if not self.ScreenActivated then
					for s = 1, #self.Screens do
						self.Screens[s] = CreateAEmitter("Att01 Download Screen");
						self.Screens[s].Pos = self.ScreenCoords[s];
						self.Screens[s]:EnableEmission(true);
						MovableMan:AddParticle(self.Screens[s]);
					end
					
					self.ScreenActivated = true;
				end
			end
		end -- for
		
		-- Check boot status
		if not isinside then
			-- Reset all completion signs if we're not inside the terminal zone
			self:AddObjectivePoint("Security Terminal", self.Terminal:GetCenterPoint() + Vector(0 , - 25), CF_PlayerTeam, GameActivity.ARROWDOWN);
			if MovableMan:IsParticle(self.Screen) then
				MovableMan:RemoveParticle(self.Screen);
				self.Screen = nil;
			end
			
			self.ChunksComplete = 0;
			self.ScreenActivated = false;
			self.HackTimer:Reset();
		else
			self:AddObjectivePoint(string.format("Hacking: %d%%", self.ChunksComplete / self.TotalChunks * 100), self.Terminal:GetCenterPoint() + Vector(0 , - 25), CF_PlayerTeam, GameActivity.ARROWDOWN);

			if self.HackTimer:IsPastSimMS(self.ChunkInterval) then
				self.ChunksComplete = self.ChunksComplete + 1;

				if self.ChunksComplete == self.TotalChunks then
					print ("Hacked!");
					for s = 1, #self.Screens do
						MovableMan:RemoveParticle(self.Screens[s]);
						self.Screens[s] = nil;
						self.Screens[s] = CreateAEmitter("Att01 Download Screen Done");
						self.Screens[s].Pos = self.ScreenCoords[s];
						self.Screens[s]:EnableEmission(true);
						MovableMan:AddParticle(self.Screens[s]);
					end
					self.TerminalHacked = true;
					self:ReplaceCapturedObjects();
				end

				self.HackTimer:Reset();
			end
		end -- if - else
	end
end
-----------------------------------------------------------------------------------------
-- Prepare everything for counter attack
-----------------------------------------------------------------------------------------
function RoninLastStand:ReplaceCapturedObjects()
	print ("ReplaceCapturedObjects");
	local toremove;
	local lpos;

    for actor in MovableMan.Actors do
		toremove = false; 
		
		if actor.Team == CF_CPUTeam then
			if string.find(actor.PresetName, "SAM Site") then
				lpos = Vector(actor.Pos.X , actor.Pos.Y);
				actor.Pos = Vector(-100 , 2000);
				toremove = true;
				
				local newactor = nil;
				
				if MovableMan:GetMOIDCount() < CF_MOIDLimit then
					newactor = CreateActor("SAM Site" , "UnmappedLands.rte");
					newactor.AIMode = Actor.AIMODE_SENTRY;
					newactor.Team = CF_PlayerTeam;
					newactor.Pos = lpos;
					MovableMan:AddActor(newactor);
				end
			end
			
			if string.find(actor.PresetName, "Small MG Turret") then
				lpos = Vector(actor.Pos.X , actor.Pos.Y);
				actor.Pos = Vector(-100 , 2000);
				toremove = true;
				
				local actor = nil;
				
				if MovableMan:GetMOIDCount() < CF_MOIDLimit then
					newactor = CreateACrab("Small MG Turret" , "Dummy.rte");
					newactor.AIMode = Actor.AIMODE_SENTRY;
					newactor.Team = CF_PlayerTeam;
					newactor.Pos = lpos;
					MovableMan:AddActor(newactor);
				end
			end
		end
		
		if toremove then 
			actor.Lifetime = 1
			ActivityMan:GetActivity():ReportDeath(actor.Team,-1)
		end
    end
end
-----------------------------------------------------------------------------------------
-- Prepare everything for counter attack
-----------------------------------------------------------------------------------------
function RoninLastStand:StartCounterAttack()
	print ("StartCounterAttack");
	self.CounterAttackTimer:Reset();
    self.CurrentFightPhase = self.FightPhase.COUNTER;
	self.DropshipSpawnInterval = self.CounterDropshipSpawnInterval;
end
-----------------------------------------------------------------------------------------
-- Selects LZ according to collected stats
-----------------------------------------------------------------------------------------
function RoninLastStand:SelectLZ()
	-- First determine if we need to help attacked base
	-- Bases closer to brain are more prioritized
	for  i = #self.Bases , 0 do
		if self.BaseTaken[i] == false then
			if self.BaseAttacked[i] == true then
				return i;
			end
		end
	end

	-- Now distribute troops to less fortified bases
	local mintroops = 100000;
	local mintroopsbase = 0;

	-- Find base with minimum of troops
	for  i = 1 , #self.Bases do
		if self.BaseTaken[i] == false then
			if self.BaseGarrison[i] < mintroops then
				mintroops = self.BaseGarrison[i]
				mintroopsbase = i
			end
		end
	end
	
	return mintroopsbase;
end
-----------------------------------------------------------------------------------------
-- Spawn dropship. Returns true if spawn is successfull, false otherwise
-----------------------------------------------------------------------------------------
function RoninLastStand:SpawnDropship()
	if self.DropshipSpawnTimer:IsPastSimMS(self.DropshipSpawnInterval) then
		if MovableMan:GetMOIDCount() < CF_MOIDLimit - 20 then
			local ship = nil;
			local actor = nil;
			local price = 0;
			local totalprice = 0;
			local p;
			local landpos = nil;
			local lznum;
			local spawnhunters = false;
			
			if self.CurrentFightPhase == self.FightPhase.COUNTER then
				if self.PlayerBrainBase > 0 and self.PlayerBrainBase < 4 then
					landpos = Vector(self.RetakeLZ[self.PlayerBrainBase]:GetRandomPoint().X , -10);
				else
					for player = 0, self.PlayerCount - 1 do
						if MovableMan:IsActor(self.AssignedBrains[player]) then
							landpos = Vector(self.AssignedBrains[player].Pos.X , -10);
							break;
						end
					end
				end
			else
				landpos = Vector(self.EnemyLZ[self:SelectLZ()]:GetCenterPoint().X , -10);
			end
			
			if landpos == nil then 
				return;
			end
			
			ship , price = CF_SpawnDropship(CF_CPUTeam , landpos , CF_CPUFaction);
			totalprice = totalprice + price;
			
			local aimode;
			
			if self.CurrentFightPhase == self.FightPhase.COUNTER then
				aimode = Actor.AIMODE_BRAINHUNT;
			else
				aimode = Actor.AIMODE_SENTRY;
			end
		
			-- Spawn armor or regular troopers depending on turn number to increase difficulty
			if math.random() < 0.15 then
				-- Give single dreadnought as dummy armor or two assalut troopers with shields
				-- to any other faction
				if CF_CPUFaction == CF_Factions.DUMMY then
					actor , price = CF_SpawnArmor(CF_CPUTeam , nil , CF_CPUFaction , aimode);
					totalprice = totalprice + price;
					
					ship:AddInventoryItem(actor);
				else
					for i = 1, 2 do
						actor , price = CF_SpawnArmor(CF_CPUTeam , nil , CF_CPUFaction , aimode);
						totalprice = totalprice + price;

						ship:AddInventoryItem(actor);
					end
				end
			else
				for i = 1, 2 do
					actor , price = CF_SpawnRandomInfantry(CF_CPUTeam , nil , CF_CPUFaction , aimode);
					totalprice = totalprice + price;

					ship:AddInventoryItem(actor);
				end
			end

			MovableMan:AddActor(ship);
		end 
		self.DropshipSpawnTimer:Reset();
	end --]]--
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function RoninLastStand:UpdateActivity()
   	self:ClearObjectivePoints();
	
	if not self.GenericTimer:IsPastSimMS(1500) then
		for player = 0, self.PlayerCount - 1 do
			self:ResetMessageTimer(player);
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("ELEMINATE ENEMY COMMANDER!", player, 500, 10000, true);
		end
	end

	if not self.GenericTimer:IsPastSimMS(5000) then
		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				self:AddObjectivePoint("Protect Commander", self.AssignedBrains[player].AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
	end
	
	if self.CurrentFightPhase == self.FightPhase.COUNTER then
		if not self.CounterAttackTimer:IsPastSimMS(7000) then
			for player = 0, self.PlayerCount - 1 do
				FrameMan:ClearScreenText(player);
				FrameMan:SetScreenText("SURVIVE COUNTERATTACK!!!", player, 0, 5000, true);
			end
		end
		
		if self.CounterAttackTimer:IsPastSimMS(40000) then
			print ("Counterattack ended");
			self.CurrentFightPhase = self.FightPhase.NOFIGHT;
			
			self.DropshipSpawnInterval = self.DefaultDropshipSpawnInterval;
		end
	else
		for i = 1 , 3 do
			if self.BaseTaken[i] == false then
				self:AddObjectivePoint("Take Over!", self.BaseArrowsCoords[i], CF_PlayerTeam, GameActivity.ARROWDOWN);	
			end
		end
	end

	if not self.TerminalHacked then
		self:HackTerminal();
	end
	
	self:UpdateUnitStats();
	self:DistributeReinforcements();
	self:SpawnDropship();
	
	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReturnOnMissionEnd();
	CF_CheckForeignTroops();
	-- end of generic part
	---------------------------------------------------------------------
	
	self:DoBrainSelection();
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function RoninLastStand:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);
	
			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				self:SetPlayerBrain(nil, player);
				local newBrain = MovableMan:GetUnassignedBrain(team);
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					self.braindead[player] = true;
					
					if not MovableMan:GetClosestBrainActor(team) then
						if CF_Activity.ActivityState ~= Activity.OVER then
							---------------------------------------------------------------------
							-- Mission framework generic part. Must present in every mission!
							CF_CommitDefeat(self.Config , self.ModuleName , self.ConfigName);
							-- end of generic part
							---------------------------------------------------------------------
							self.WinnerTeam = self:OtherTeam(team);
							ActivityMan:EndActivity();
						end
					end
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Enter orbit, check winning conditions
-----------------------------------------------------------------------------------------
function RoninLastStand:CraftEnteredOrbit()
	print (CF_BrainPresets[CF_PlayerFaction]);
	if self.OrbitedCraft:HasObject(CF_BrainPresets[CF_PlayerFaction]) then
		if CF_Activity.ActivityState ~= Activity.OVER then
			---------------------------------------------------------------------
			-- Mission framework generic part. Must present in every mission!
			CF_CommitRetreat(self.Config , self.ModuleName , self.ConfigName);
			-- end of generic part
			---------------------------------------------------------------------
			self.WinnerTeam = CF_CPUTeam;
			ActivityMan:EndActivity();
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































